/* umtsmon - a program to control/monitor your UMTS card
 * Copyright (C) 2007,2008  Klaas van Gend
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * applicable version is GPL version 2 only.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef NETWORKSTATS_H_
#define NETWORKSTATS_H_

#include "MonthlyTraffic.h"

#include <qwidget.h>
#include <qlabel.h>
#include <qpixmap.h>
#include <qfont.h>
#include <qcolor.h>

/// This GUI class graphically displays various information for the user
class LCDDisplay : public QLabel
{
	Q_OBJECT
	
public:
	LCDDisplay( QWidget * aParent, const char * aName = 0, WFlags aFlags = 0 );

	/// sets the operatorname and updates the GUI 
	void setOperatorName(const QString& aName, bool needsUpdate=true) 
		{setStringAndUpdate(theOperatorName, aName, needsUpdate);};
	/// sets the radio type string (HSDPA, UMTS, GPRS, etc) and updates the GUI 
	void setRadioType(const QString& aName, bool needsUpdate=true)
		{setStringAndUpdate(theRadioType, aName, needsUpdate);};
	/// sets the profile name and updates the GUI 
	void setProfile(const QString& aName, bool needsUpdate=true)
		{setStringAndUpdate(theProfile, aName, needsUpdate);};
	
	/// sets the upload string and updates the GUI 
	void setUpload(unsigned long aNumber, bool needsUpdate=true)
		{setStringAndUpdate(theUpload, number2HumanString(aNumber), needsUpdate);};
	/// sets the download string and updates the GUI 
	void setDownload(unsigned long aNumber, bool needsUpdate=true)
		{setStringAndUpdate(theDownload, number2HumanString(aNumber), needsUpdate);};

	/// sets the monthly total traffic and update the GUI
	void setTotalTraffic(unsigned long aNumber, MonthlyTraffic::ThresholdStatus aStatus, bool needsUpdate=true)
		{
			theTotalStatus = aStatus;
			setStringAndUpdate(theTotal, number2HumanString(aNumber), needsUpdate);
		};


	/** sets the signal strength of the radio (result of AT+CSQ)
	 *  @param aNewStrength  values of 0-31 and 99 are allowed
	 */
	void setStrength(unsigned int aNewStrength, bool needsUpdate=true);

protected:
	virtual void drawContents ( QPainter * p ); 
	
private:
	/** internal function that will set a string to a name and make the widget
	 *  update its looks if needed
	 *  @param aString OUTPUT the qstring to put stuff in to
	 *  @param aName   INPUT the qstring with the name to set
	 *  @param needsUpdate  true if you want to redraw the widget, false if not (yet)
	 */ 
	void setStringAndUpdate(QString& aString, const QString& aName, bool needsUpdate)
		{ aString = aName.left(17); if (needsUpdate) {this->update();}; };

	/** converts an ulong into a human readable number string.
	 *  i.e.       31 will become 31 B
	 *           3141 will become 3.07 KiB
	 *         314156 will become 307 KiB
	 *       31415654 will become 30.0 MiB
	 *  @param aNumber the number to convert
	 *  @returns the converted and human readable string
	 */
	QString number2HumanString(unsigned long aNumber) const;

	QPixmap theGaugePixmap;
	
	/** this is the width of theGaugePixmap to draw (max 104, min 0 in 20 steps)
	 *  -1 means "draw n/a"
	 */
	int	theSignalWidth;
	
	QString	theOperatorName;
	QString theRadioType;
	QString theProfile;
	
	QString theUpload;
	QString theDownload;
	QString theTotal;
	MonthlyTraffic::ThresholdStatus theTotalStatus;
	
	// three font sizes to cache 
	QFont   the14Font;
	QFont   the10Font;
	QFont   the8Font;
	
	// two colors to cache
	QColor  theBlack;
	QColor  theShadow;
};


#endif /*NETWORKSTATS_H_*/
