/* umtsmon - a program to control/monitor your UMTS card
 * Copyright (C) 2006, 2007  Klaas van Gend, Christofer Wesseling
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * applicable version is GPL version 2 only.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you want to add, delete, or rename functions or slots, use
** Qt Designer to update this file, preserving your code.
**
** You should not define a constructor or destructor in this file.
** Instead, write your code in functions called init() and destroy().
** These will automatically be called by the form's constructor and
** destructor.
*****************************************************************************/
#include "umtsmon_global.h"
#include "assert.h"
#include "NetworkChanger.h"
#include "Popup.h"

void networkoperator::init()
{
	theNWChangerPtr = NULL;
}

void networkoperator::destroy()
{
	if (theNWChangerPtr != NULL)
	{
		delete theNWChangerPtr;
		theNWChangerPtr = NULL;
	} 
}


void networkoperator::SearchNetworkOperator()
{
	/* actions to do:
	 *  1) display "searching" dialog and start search
	 *  2) enable the listbox and the textLabel about double click
	 *  3) parse the output from the search and display it
	 */
	assert(theNWChangerPtr == NULL);
	theNWChangerPtr = new NetworkChanger;
	if (theNWChangerPtr->startSearching() == false)
	{
		// we get here if the card immediately returned with an error,
		// e.g. if the PIN code has not been entered or 
		// if the PIN has not propagated through all of the card's subsystems 
		// yet - see Bug # 1527401.
		//
		// for now - fail silently expected user behaviour is to click the
		// button again after approx 2 secs as nothing happened... 
		emit StopSearchNWOperator();
		return;
	}
	
	const int MAXTIMEOUT = 180;
	// for as long as detection and selection takes, let's disable the
	// 'find networks' button (see bug # 1560481)
	pushButtonFindNetworks->setEnabled(false);

	// detection process can take a while...
	// wait for completion with a progress window...
	ProgressDialog myProgress(this,  tr("Searching for mobile networks..."));
	// detection will take time anyway, so better show progress from start
	myProgress.start(0);
	myProgress.setMaxTime(MAXTIMEOUT);
	while (!theNWChangerPtr->isSearchCompleted())
	{
		if (myProgress.runOneSecond() == false)
		{
			// Apparently, the user pressed cancel.
			// Let's propagate that to the main window for cleaning purposes.
			emit StopSearchNWOperator();
			this->reject(); //lets close the dialog... user dind't want wo look for network operators..
			return;
		}
	}
	myProgress.finish();

	// if we have time left, we apparently got success
	QPixmap myOK( QPixmap::fromMimeSource("ok.png") );
	QPixmap myForbidden( QPixmap::fromMimeSource("process-stop-16x16.png") );
	QPixmap myNull;
	networkPicker->clear();

	if (theNWChangerPtr->theCNlist.empty()==true)
	{
		Popup::Info(tr("No network operators were found.\nYou might want to try again."), this);
		emit this->reject();
		return;
	}

	for (NetworkChanger::CNlist::iterator myI=theNWChangerPtr->theCNlist.begin();
		myI != theNWChangerPtr->theCNlist.end(); myI++)
	{
		QPixmap* myPMPtr = NULL;
		switch((*myI)->theIconType)
		{
		case NetworkChanger::FORBIDDEN:
			myPMPtr = &myForbidden;
			break;
		case NetworkChanger::AVAILABLE:
			myPMPtr = &myNull;
			break;
		case NetworkChanger::CURRENT:
			myPMPtr = &myOK;
			break;
		}
		QString myName = (*myI)->theNetworkName
			 + "                                       *" +
			   QString::number((*myI)->theNetworkID);
		networkPicker->insertItem(*myPMPtr, myName);
	}

	networkPicker->setEnabled(true);
	pushButtonSelect->setEnabled(true);
	networkPickerComments->setText(tr("Select your operator from the above list."));
}


// this SLOT is called whenever a switch network has been completed
// or cancelled. It returns everything to its regular state. 
void networkoperator::StopSearchNWOperator()
{
	assert(theNWChangerPtr != NULL);
	delete theNWChangerPtr;
	theNWChangerPtr = NULL;

	pushButtonFindNetworks->setEnabled(true);
	networkPicker->setEnabled(false);
	networkPickerComments->setText("");
}


void networkoperator::switchToNetworkID( int aIndex )
{
	assert(theNWChangerPtr != NULL);
	aIndex = networkPicker->currentText().section("*",1,1).toLong();
	DEBUG2("requested switchToNetworkID: %d\n", aIndex);

	// if forbidden, put up a message to say so
	if (theNWChangerPtr->startSwitchToNetworkID(aIndex) == false)
	{
		Popup::Warning (tr(APPNAME " cannot connect.\n"
			"Either this network is forbidden to you or an error happened.\n"
			"Please select a different network or retry."),
			this);
		return;
	}

	// connection process can take a while...
	// wait for completion with a progress window...
	ProgressDialog myProgress(this, tr("Connecting to mobile network...")); 
	myProgress.setMaxTime(100);
	// connection will take time anyway, so better show progress from start
	myProgress.start(0);
	while (!theNWChangerPtr->isSwitchCompleted())
	{
		if (myProgress.runOneSecond() == false)
		{
			// Apparently, the user pressed cancel.
			// let's quit the while and start cleaning
			break;
		}
	}
	myProgress.finish();

	// and.. we're done
	// clean out everything
	emit StopSearchNWOperator();
}


void networkoperator::SelectCurrentNetworkOperator()
{
    switchToNetworkID( networkPicker->currentItem() );
    this->accept();
}
